#include <Adafruit_Sensor.h>
#include "DHT.h"
#include <TinyGPS.h>
#include <SoftwareSerial.h>
#include <Arduino.h>
#include <U8x8lib.h>
#include <Wire.h>
#include <SPI.h>

U8X8_SSD1306_128X64_NONAME_SW_I2C u8x8(/* clock=*/ SCL, /* data=*/ SDA, /* reset=*/ U8X8_PIN_NONE);   // OLEDs without Reset of the Display

static const int RXPin = A3, TXPin = A2;
static const uint32_t GPSBaud = 9600;
TinyGPS gps;
SoftwareSerial ss(RXPin, TXPin);
#define DHTPIN 7  // The DHT11 object
#define DHTTYPE DHT11   // DHT 11
DHT dht(DHTPIN, DHTTYPE);


unsigned long  sys_clk  = 16000000;
unsigned long  frq_init = 52000;
unsigned long  frq_min = 20000;
unsigned long  frq_max = 99900;
unsigned long  frq;
unsigned int   sonic_pp = 0;
unsigned int   sonic_prk = 0;
unsigned int  sonic_adj = 0;
unsigned int Level_R, Level_L;
float gps_lat, gps_lng;
bool newData = false;
float hum, temp;

int frqset_frg = 0;
String buildJson() ;


void setTCR1_frq(unsigned long frq) {
  unsigned int fdata;
  fdata = sys_clk / frq;
  ICR1 = fdata;
  OCR1A = fdata / 2; // PWM 50%
  OCR1B = fdata / 2; // PWN 50%
}

void contrl_ph_A() {
  int pha_st, phb_st;
  if (frqset_frg == 0) {
    frqset_frg = 0;
    pha_st = digitalRead(2);
    phb_st = digitalRead(3);
    if ((pha_st == HIGH) && (phb_st == HIGH)) {
      frq = frq + 100;
    }
    else if ((pha_st == LOW) && (phb_st == HIGH)) {
      frq = frq + 100;
    }
    else if ((pha_st == HIGH) && (phb_st == HIGH)) {
      frq = frq - 100;
    }
    else if ((pha_st == LOW) && (phb_st == LOW)) {
      frq = frq - 100;
    }
    else {
    }
    if(frq > frq_max) frq = frq_max;
    else if (frq < frq_min ) frq =  frq_min;
    setTCR1_frq(frq);
  }
  else {
  }

}

void contrl_ph_B() {
  int pha_st, phb_st;
  if (frqset_frg == 0) {
    frqset_frg = 0;
    pha_st = digitalRead(2);
    phb_st = digitalRead(3);
    if ((phb_st == HIGH) && (pha_st == HIGH)) {
      frq = frq - 100;
    }
    else if ((phb_st == LOW) && (pha_st == HIGH)) {
      frq = frq - 100;
    }
    else if ((phb_st == HIGH) && (pha_st == HIGH)) {
      frq = frq + 100;
    }
    else if ((phb_st == LOW) && (pha_st == LOW)) {
      frq = frq + 100;
    }
    else {
    }
    if(frq > frq_max) frq = frq_max;
    else if (frq < frq_min ) frq =  frq_min;
    setTCR1_frq(frq);
  }
  else {
  }
}
unsigned int getSonicLevel_R(void) {
  unsigned int   sonic_level;
  unsigned int   sonic_max = sonic_adj;
  unsigned int   sonic_min = sonic_adj - 1 ;

  //  for (int i = 0; i < 1024; i++) {
  for (int i = 0; i < 256; i++) {
    sonic_level = analogRead(A0);
    if (sonic_max < sonic_level) sonic_max = sonic_level;
    else if (sonic_min > sonic_level) sonic_min = sonic_level;
  }
  return (sonic_max - sonic_min);
}

unsigned int getSonicLevel_L(void) {
  unsigned int   sonic_level;
  unsigned int   sonic_max = sonic_adj;
  unsigned int   sonic_min = sonic_adj - 1 ;

  //  for (int i = 0; i < 1024; i++) {
  for (int i = 0; i < 256; i++) {
    sonic_level = analogRead(A1);
    if (sonic_max < sonic_level) sonic_max = sonic_level;
    else if (sonic_min > sonic_level) sonic_min = sonic_level;
  }
  return (sonic_max - sonic_min);
}

void get_Sonic_Level(unsigned int *Level_R, unsigned int *Level_L) {
  *Level_R = 0;
  for (int i = 0; i < 64; i++) {
    *Level_R = *Level_R + getSonicLevel_R();
    delay(0.1);
  }
  *Level_R = *Level_R >> 6;

  *Level_L = 0;
  for (int i = 0; i < 64; i++) {
    *Level_L = *Level_L + getSonicLevel_L();
    delay(0.1);
  }
  *Level_L = *Level_L >> 6;
}

void setup() {
  pinMode(9, OUTPUT); // 0C1A
  pinMode(10, OUTPUT);// 0C1B
  pinMode(2, INPUT);
  pinMode(3, INPUT);
  pinMode(4, INPUT_PULLUP);

  attachInterrupt(0, contrl_ph_A, CHANGE);
  attachInterrupt(1, contrl_ph_B, CHANGE);

  ADCSRA = ADCSRA & 0xf8;
  ADCSRA = ADCSRA | 0x04;
  for (int i = 0; i > 16 ; i++) {
    sonic_adj = sonic_adj + analogRead(A0);
  }
  sonic_adj = sonic_adj >> 4;

  TCCR1A = 0b10110010; // 0C1A ->10 match-Low   0C1B ->11 match-High   FastPWM-mode Top-ICR1
  TCCR1B = 0b00011001; // clock I/O/1  0.0625us
  setTCR1_frq(54000);
  frq = frq_init;
  Serial.begin(9600);
  ss.begin(GPSBaud);

  dht.begin();

  u8x8.begin();
  u8x8.setPowerSave(0);
  u8x8.setFont(u8x8_font_chroma48medium8_r);
  u8x8.drawString(0, 0, "  ULTRA SONIC");
  u8x8.drawString(0, 1, "   DETECTOR!");

}

// the loop function runs over and over again forever
void loop() {
  int serch_dir = 1;
  unsigned long f = frq_min;
  unsigned long frq_prk, frq_mix;
  sonic_prk = 0;

  Serial.println("Tunning Start");
  while (1) {
    setTCR1_frq(f);
    sonic_pp = getSonicLevel_R();
    if ( sonic_pp > sonic_prk ) {
      frq_prk = f;
      sonic_prk = sonic_pp;
    }

    if ( f < frq_init) {
      f = f + 1000;
    }
    else {
      frq_mix = frq_prk + 1000;
      frq = frq_mix;
      setTCR1_frq(frq);
      Serial.println("Tunning End");
      while (1) {

        if (digitalRead(4) == HIGH) {  // Detect Mode
          u8x8.setCursor(0, 3);
          u8x8.print("=DETECTOR MODE=");
          u8x8.setCursor(0, 4);
          u8x8.print("FRQ:");

          get_Sonic_Level(&Level_R, &Level_L);

          u8x8.setCursor(5, 4);
          u8x8.print(frq);
          u8x8.print(" Hz ");
          u8x8.setCursor(0, 5);
          u8x8.print("                ");
          u8x8.setCursor(0, 6);
          u8x8.print("L:              ");
          u8x8.setCursor(2, 6);
          for (int i = 0; i < Level_L / 8; i++) {
            u8x8.print(">");
          }
          u8x8.setCursor(0, 7);
          u8x8.print("R:              ");
          u8x8.setCursor(2, 7);
          for (int i = 0; i < Level_R / 8; i++) {
            u8x8.print(">");
          }
        }

        else {          // Record Mode
          u8x8.setCursor(0, 3);
          u8x8.print("=RECORDING MODE=");
      
          // get gps data
          // For one second we parse GPS data and report some key values
          for (unsigned long start = millis(); millis() - start < 1000;) {
            while (ss.available()) {
            char c = ss.read();
            // Serial.write(c); // uncomment this line if you want to see the GPS data flowing
            if (gps.encode(c)) // Did a new valid sentence come in?
            newData = true;
            }
          }         
          if (newData) {
            unsigned long age;
            gps.f_get_position(&gps_lat, &gps_lng, &age);
          }          
          
          u8x8.setCursor(0, 4);
          u8x8.print("FRQ:");
          u8x8.setCursor(5, 4);
          u8x8.print(frq);
          u8x8.print(" Hz ");
          u8x8.setCursor(0, 5);
          u8x8.print("LAT: ");
          u8x8.print(gps_lat, 6);
          u8x8.setCursor(0, 6);
          u8x8.print("LON:");
          u8x8.print(gps_lng, 6);

          hum = dht.readHumidity();
          temp = dht.readTemperature();

          u8x8.setCursor(0, 7);
          u8x8.print("HUM:");
          u8x8.print(hum, 1);
          u8x8.print("TMP:");
          u8x8.print(temp, 1);

          get_Sonic_Level(&Level_R, &Level_L);

          Serial.println(buildJson());
          delay(1000);
        }
      }
    }
  }
}


String buildJson() {
  String json = "{";
  json +=  "\"Device_ID\": \"Piduno01\"";
  json += ",";

  json += "\"GPS\":";
  json += "{";
  json += "\"LATITUDE\":";
  json += String(gps_lat, 6);
  json += ",";
  json += "\"LONGITUDE\":";
  json += String(gps_lng, 6);
  json += "}";
  json += ",";

  json += "\"Envroment\":";
  json += "{";
  json += "\"TEMP\":";
  json += temp;
  json += ",";
  json += "\"HUMIDITY\":";
  json += hum;
  json += "}";
  json += ",";

  json += "\"Ultrasonic\":";
  json += "{";
  json += "\"FREQ\":";
  json += frq;
  json += ",";
  json += "\"LEVEL_LEFT\":";
  json += Level_L;
  json += ",";
  json += "\"LEVEL_RIGHT\":";
  json += Level_R;
  json += "}";

  json += "}";
  return json;
}


